package oracle.forms.demos.uploadclient;

import sun.misc.BASE64Encoder;
import sun.misc.BASE64Decoder;
import java.awt.FileDialog;
import java.awt.Frame;
import java.awt.Button;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.zip.*;
import java.util.StringTokenizer;

import oracle.forms.ui.VBean;
import oracle.forms.ui.CustomEvent;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;

import oracle.ewt.meter.ProgressBar;
import oracle.ewt.meter.BoundedRangeModelImpl;
import oracle.ewt.thread.TaskScheduler;
import oracle.ewt.thread.Task;
import oracle.ewt.thread.TaskEvent;

/**
 * <P>
 * This component facilitates the sending of files from the Forms Java client to
 * the Forms server.  It allows the user to select a file to be sent to the server
 * and then performs the necessary operations required to send the file over the
 * Forms message connection to the server.
 * </P>
 * <P>
 * Since the Forms server is not able to handle byte data from it's PL/SQL environment
 * the user selected file is first transformed into a textual representation, using
 * the BASE64 encoding scheme for sending to the server.
 * </P>
 * <P>
 * To further aid in the sending of this textual data to the Forms server, the
 * encoded file is broken into 4K chunks before sending and each chunk is
 * dispatched individually.  It is up to the Forms server to process these chunks
 * and reassemble them into a useful data set.
 * </P>
 * To reduce the amount of data sent across the network the source file is
 * compressed before encoding. This facility can be turned off if the files
 * being transferred are already compressed e.g. ZIP files
 * <P>
 * Since the encoding, chunking and dispatching processes can take a significant amount
 * of time on slower networks/large files, the FileUploader utilizes a ProgressBar to
 * indicate to the end user that actions are being performed.
 * </P>
 *
 * @version 1.2
 * @date 09-1-2001
 * @author Steve Button
 * @author Duncan Mills
 */

public class FileUploader
       extends VBean
{
  /**
   * The taskscheduler provided by the EWT team to make the use of a Thread
   * to unblock the AWTEventQueue easier.
   */
  private TaskScheduler scheduler = new TaskScheduler();

  /**
   * A reference for the forms handler associated with this component.
   */
  private IHandler mHandler = null;

  /**
   * Size of the chunks to send back to form server.  Should be less
   * than the maximum length of the VARCHAR type.
   */
  private final static int CHUNKSIZE = 4096;
  /**
   * Forms property used to indicate that an upload has begun.
   */
  private static final ID UPLOADSTART = ID.registerProperty("UPLOADSTART");
  /**
   * Forms property used to indicate that an upload has ended because
   * a named file does not exist
   */
  private static final ID UPLOADABORT = ID.registerProperty("UPLOADABORT");

 /**
   * Forms property used to indicate that an upload has ended.
   */
  private static final ID UPLOADEND = ID.registerProperty("UPLOADEND");
  /**
   * Forms property used to indicate that the Forms server wants the JavaBean
   * to perform the upload process.
   */
  private static final ID LOADFILE = ID.registerProperty("LOADFILE");
  /**
   * Forms property used to indicate that the Forms server wants the JavaBean
   * to perform the upload process on a named file (no selection dialog)
   */
  private static final ID LOADNAMEDFILE = ID.registerProperty("LOADNAMEDFILE");
  /**
   * Forms property used to identify the name of the file the user has selected.
   */
  private static final ID FILENAME = ID.registerProperty("FILENAME");
  /**
   * Forms property used to identify the name of the file the user has selected.
   */
  private static final ID FILEDIRECTORY = ID.registerProperty("FILEDIRECTORY");
  /**
   * Forms property used to identify the size of the file the user has selected.
   */
  private static final ID FILESIZE = ID.registerProperty("FILESIZE");
  /**
   * Forms property used to enable or disable compression on the file
   * being transferred
   */
  private static final ID COMPRESS = ID.registerProperty("COMPRESS");
  /**
   * Forms property used to define the position of the file dialog
   */
  private static final ID DIALOGPOS = ID.registerProperty("DIALOGPOS");
  /**
   * Forms property used to identify that a piece of data is coming down the
   * pipe.
   */
  private static final ID CHUNKDATA = ID.registerProperty("CHUNKDATA");
  /**
   * Forms property used to identify the sequence of the associated data segment.
   */
  private static final ID CHUNKID = ID.registerProperty("CHUNKID");
  /**
   * Forms property used to identify a piece of encoded data from the file
   * the user has selected.
   */
  private static final ID CHUNKVALUE = ID.registerProperty("CHUNKVALUE");

  /**
   * Forms property used to indicate that we want the utility to switch
   * Messaging on to the Java Console so we can see what is going on
   * the user has selected.
   */
  private static final ID DEBUGMESSAGES = ID.registerProperty("DEBUGMESSAGES");
  /**
   * Do we want to see logging messages? (can be set dynamically using
   * DEBUGMESSAGES
   */
  private static boolean DEBUG = false;


  /**
   * Is compression enabled?
   */
  private boolean mCompression = true;

 /**
  *  Hold the position for the File Dialog
  */

  private int mXPos = 40;
  private int mYPos = 20;

  /**
   * The progress bar we use as a visual indicator
   */
  private FileUploaderProgressBar progressBar = null;
  /**
   * Stores the previous directory selected.
   */
  private String dirSelect = null;

  /**
   * Creates a new instance of the FileUploader class.
   */
  public FileUploader()
  {
    log(getClass().getName());
  }

  /**
   * Method from the oracle.forms.ui.IView class and overriden from the VBean
   * base class.  This lifecycle method is called once when Forms creates an
   * instance of the JavaBean.
   *
   * @param <b>handler</b> a reference to the Handler class for this JavaBean.
   */
  public void init(IHandler handler)
  {
    mHandler = handler;
  }

  /**
   * Overriding method from the IView interface.  Allows the values of properties
   * to be retrieved when called from the Oracle Forms Server runtime.  This method
   * allows the Forms Server to get both standard and custom properties.  The standard
   * properties such as FOREGROUND, FONT, etc. all have type mappings performed
   * by the Forms engine itself so these can simply be passed from their analog
   * accessor methods on the JavaBean.   For the custom properties, sometimes
   * type translation is required since Forms will only allow a String
   * type as a return value for a custom property.
   *
   * @param <b>pid</b> the property to be retrieved, identified by it's ID
   * @param <b>Object</b> the value of the requested property
   * @return The outcome of the operation.
   */
  public boolean setProperty(ID pid, Object value)
  {
    if(pid == LOADFILE)
    {
      return loadFileWithProgress(value,false);
      //return loadFile(value,false);
    }
    else if(pid == LOADNAMEDFILE)
    {
      return loadFileWithProgress(value,true);
    }
    else if (pid == DEBUGMESSAGES)
    {

      if (value.toString().equalsIgnoreCase("true"))
        DEBUG = true;
      else
        DEBUG = false;
    }
    else if (pid == COMPRESS)
    {

      if ((value instanceof String) && value.toString().equalsIgnoreCase("true"))
        mCompression = true;
      else
        mCompression = false;
    }
    else if (pid == DIALOGPOS)
    {
      StringTokenizer posTokens = new StringTokenizer(value.toString(),",");
      log("Setting Position of File dialog to : " + value.toString());
        mXPos = Integer.parseInt(posTokens.nextToken());
        mYPos = Integer.parseInt(posTokens.nextToken());
    }

    return super.setProperty(pid, value);
  }

  /**
   * This is a special method required to allow the progress bar to update itself
   * by running the long running loadFile method in a separate thread.
   *
   * @param <b>value</b> The value that was passed in with the LOADFILE or LOADNAMEDFILE property.
   * @param <b>namedFile</b> Indicator that the named file should just be loaded without displaying
   * a file dialog to allow the user to choose.
   * @return The result of the loadFile method (always TRUE).
   */
  private boolean loadFileWithProgress(Object value, boolean namedFile)
  {
    final Object val = value;
    final boolean nf = namedFile;

    new FileUploaderLoadThread(this,val,nf).start();

    // force a garbage collection ...
    System.gc();

    return true;
  }

  /**
   * Facilitates the sending of a user selected file back to the Forms Server
   * encoded using the BASE64 encoding scheme.<BR>
   * Enables the user to make a file selection using the standard AWT file dialog.<BR>
   * Optionally Zips the file<BR>
   * Encodes the file using the sun.misc.BASE64Encoder class.<BR>
   * Sends the encoded data back in segments of size CHUNKSIZE.
   *
   * @param <b>value</b> sent with the LOADFILE or LOADNAMEDFILE property from
   * the Forms Server, used to indicate the starting directory the filedialog
   * should use if the second argument is false, or the name of the actual file
   * to load (bypassing dialog) if the second argument is true
   * @param <b>namedFile</b> indicates that the supplied value is for a specific file to upload.
   * @return The outcome of the operation.
   */
  protected boolean loadFile(Object value, boolean namedFile)
  {
    int len = 0;
    int offset = 0;
    int leftover = 0;
    String chunk = null;
    String data = null;
    Frame f = new Frame("");
    FileDialog fd = new FileDialog(f);
    String fileName = null;
    String fileDirectory = null;

    // First of all chack to see if we need to pop up a file dialog or not

    if (!namedFile)
    {
    // Check to see if a value was sent with this property to indicate
    // the starting directory the filedialog should use.
    //
    // If not, use the previous directory selected - or null if it is the
    // first time
    //
      log("LoadFile : File Selection");
      if(value != null && value instanceof String)
        fd.setDirectory((String)value);
      else
        fd.setDirectory(dirSelect);

    // Display the filedialog.
      fd.setLocation(mXPos,mYPos);
      fd.show();
      if(fd.getFile() != null)
      {
        fileName = fd.getFile();
        fileDirectory = fd.getDirectory();
        log("Selected File = " + fileDirectory + fileName);
      }
    }
    else
    {
      log("LoadFile : Named File");
      if(value != null && value instanceof String)
      {
        File fl = new File((String)value);
        if (fl.canRead())
        {
          fileName = fl.getName();
          log(fileName);
          fileDirectory = fl.getParent() + fl.separator;
          log("Dir>" + fileDirectory);
          log("Named File = " + fileDirectory + fileName + " exists");
        }
        else
        {
          dispatchError("Named File " + (String)value + " does not exist or cannot be read");
        }
        fl = null;
      }
    }

    if (fileName != null)
    {
      // Send message to forms indicating we are starting the upload and the
      // name of the selected file.
      dispatchMessage(UPLOADSTART);
      dispatchFilename(fileDirectory, fileName);

      log("<PERFORM UPLOAD>");

      try
      {
        // set the progress bar with some information
        progressBar = new FileUploaderProgressBar(f);
        progressBar.setMessage("Encoding " + fileName);
        progressBar.setSteps(1);
        progressBar.setVisible(true);

        // keep track of the selected directory
        dirSelect = new String(fileDirectory);

        // Optionally compress the selected file and then
        // encode using the BASE64 mechanism
        data = encodeBuffer(fileDirectory + fileName,mCompression);
        progressBar.increment();
        progressBar.setVisible(false);

        len = data.length();
        log("Size of encoded byte array : " + data.length() + " (" + ((data.length()/CHUNKSIZE) + 1)  + " Chunks)");

        // Now send over the data, set dialog message appropriately
        progressBar.setMessage("Uploading " + fileName);

        // Since the data chunks are handled in forms using a VARCHAR type, the
        // chunks can be of a maximum 32K in length.  If the length of the encoded
        // data is bigger than the chunksize, split it and then send it.
        if(len > CHUNKSIZE)
        {
          int numchunks = (int)len / CHUNKSIZE ;
          progressBar.setSteps(numchunks);
          progressBar.setVisible(true);

          for (int i=0;i<numchunks;i++)
          {
            progressBar.setValue(i);
            offset = i*CHUNKSIZE;
            leftover = CHUNKSIZE;
            chunk = data.substring(offset,offset+leftover);
            log("Chunking : " + offset + ":" + (offset+leftover));
            dispatchChunk(i,chunk);
          }

          // If there is any left over data then send it
          progressBar.setValue(numchunks);
          offset = numchunks*CHUNKSIZE;
          leftover = len;
          chunk = data.substring(offset,len);
          log("Chunking : " + offset + ":" + len);
          dispatchChunk(numchunks,chunk);
        }
        // The length of the data is less than the chunksize so send it all
        // at once.
        else
        {
          progressBar.setSteps(1);
          progressBar.setMessage("Uploading " + fileName);
          progressBar.setValue(1);;
          dispatchChunk(0,data);
        }
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }

      // Send a message to indicate that the upload process has ended and
      // hide progress bar
      progressBar.setVisible(false);
      dispatchMessage(UPLOADEND);

      log("</PERFORM UPLOAD>");
    }

    // Clean up the variables.
    data=null;
    chunk=null;
    fileName=null;
    fileDirectory=null;
    return true;
  }

  /**
   * Convenience routine to dispatch an event back to forms.
   *
   * @param <b>pid</b> the property to indicate which event has occurred.
   */
  private void dispatchMessage(ID pid)
  {
    try
    {
      CustomEvent ce = new CustomEvent(mHandler,pid);
      super.dispatchCustomEvent(ce);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Convenience routine to send the filename the user has selected back
   * to Forms.
   *
   * @param <b>fileDirectory</b> The directory of the selected file
   * @param <b>fileName</b> The name of the selected file
   */
  private void dispatchFilename(String fileDirectory, String fileName)
  {
    try
    {
      CustomEvent ce = new CustomEvent(mHandler,FILENAME);
      mHandler.setProperty(FILENAME,fileName);
      mHandler.setProperty(FILEDIRECTORY,fileDirectory);
      super.dispatchCustomEvent(ce);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
 /**
   * Convenience routine to send the filesize of the file the user has selected
   * back to Forms.
   *
   * @param <b>fileSize</b>The size of the selected file
   */
  private void dispatchFilesize(int fileSize)
  {
    log("Filesize " + fileSize);
    try
    {
      CustomEvent ce = new CustomEvent(mHandler,FILESIZE);
      mHandler.setProperty(FILESIZE,"" + fileSize);
      super.dispatchCustomEvent(ce);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /* Convenience routine to send back an Abort to Forms
   *
   * @param <b>errorMsg</b>The size of the selected file
   */
    private void dispatchError(String errorMsg)
    {
      log(errorMsg);
      try
      {
        CustomEvent ce = new CustomEvent(mHandler,UPLOADABORT);
        mHandler.setProperty(UPLOADABORT,errorMsg);
        super.dispatchCustomEvent(ce);
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
  }

  /**
   * Convenience routine to send a chunk of data back to forms.
   *
   * @param <b>chunkum</b> the sequence number of the chunk.
   * @param <b>chunk</b> the data.
   */
  private void dispatchChunk(int chunknum, String chunk)
  {
    log("Dispatch Chunk " + (chunknum + 1));
    Integer i = new Integer(chunknum);

    try
    {
      CustomEvent ce = new CustomEvent(mHandler,CHUNKDATA);
      mHandler.setProperty(CHUNKID,i.toString());
      mHandler.setProperty(CHUNKVALUE,chunk);
      super.dispatchCustomEvent(ce);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Reads the file specified, compresses it and converts it into a BASE64 encoded string
   * representation.
   *
   * @param <b>filename</b> the name of the file to encode.
   * @param <b>compress</b> indicates that the file should be zipped before encoding
   * @return a string containing the BASE64 encoded file.
   */
  private String encodeBuffer(String filename, boolean compress) throws java.io.FileNotFoundException, java.io.IOException
  {
    FileInputStream fis = new FileInputStream(filename);
    byte data[] = new byte[fis.available()];
    fis.read(data);
    dispatchFilesize(data.length);

    if (compress)
    {
      ByteArrayOutputStream compOs = new ByteArrayOutputStream();
      GZIPOutputStream gos = new GZIPOutputStream( compOs );
      gos.write(data, 0, data.length );
      gos.finish();
      gos.close();
      byte [] compressedData = compOs.toByteArray();
      int compressedLength = new String(compressedData).trim().length();
      log("Source file size before encoding " + data.length + " Bytes; (Compressed to " + compressedLength + " Bytes)");
      data = new byte[compressedLength];
      System.arraycopy(compressedData,0,data,0,compressedLength);
    }
    else
    {
      log("Source file size before encoding " + data.length + " Bytes");
    }

    BASE64Encoder encoder = new BASE64Encoder();

    return encoder.encodeBuffer(data);
  }

  /**
   * Writes debugging messages to the Java Console when the DEBUGMESSAGES property has been set on the bean.
   *
   * @param <b>msg</b> the console output
   */

  public void log(String msg)
  {
    if(DEBUG)
      System.out.println(getClass().getName() + ": " + msg);
  }

  public static void main(String[] args)
  {
    System.out.println("Test Mode");
    FileUploader fileUploader = new FileUploader();
    fileUploader.setProperty(fileUploader.LOADFILE,"C:\\temp");
    System.exit(0);

  }

}

