package oracle.forms.demos.uploadclient;

import oracle.ewt.meter.*;
import java.awt.*;

/**
 * <P>
 * This component provides a Progress indicator for the FileUploader to
 * give some immediate feedback to the end user that something is actually
 * happening.
 * </P>
 *
 * @version 1.0
 * @date 08-11-2000
 * @author Steve Button
 */

public class FileUploaderProgressBar extends Dialog
{
  private Frame frame = null;
  private ProgressBar progressBar = null;
  private BoundedRangeModel model = null;
  private Label message = null;
  private Panel mPanel = null;
  private boolean progressEnabled = true;

  private static final int WIDTH = 400;
  private static final int HEIGHT = 100;
  private static final String TITLE = "File Upload";
  private static final String DIALOG_TEXT = "Upload Progress";
  private static final Color DIALOG_COLOR = Color.decode("#002288");

  /**
   * Constructor
   */
  public FileUploaderProgressBar(Frame owner)
  {
    super(owner);
    frame = owner;

    setResizable(false);
    setTitle(TITLE);
    setSize(WIDTH,HEIGHT);
    setLayout(new BorderLayout());

    model = new BoundedRangeModelImpl(0,1,0);
    progressBar = new ProgressBar(model);
    message = new Label("");

    Panel tPanel = new Panel();
    tPanel.setLayout(new GridLayout());
    Label tLabel = new Label();
    Font tFont = new Font("Helvetica",Font.BOLD,14);
    tLabel.setFont(tFont);
    tLabel.setText(DIALOG_TEXT);
    tLabel.setForeground(DIALOG_COLOR);
    tPanel.add(tLabel);

    mPanel = new Panel();
    mPanel.add(message);

    add(tPanel,BorderLayout.NORTH);
    add(mPanel,BorderLayout.SOUTH);
    add(progressBar, BorderLayout.CENTER);

    setLocation(getCenteredLocation(owner));
  }

  public void setSteps(int val)
  {
    model = new BoundedRangeModelImpl(0,val,0);
    progressBar.setModel(model);
  }

  public void setMessage(String msg)
  {
    message.setText(msg);
    mPanel.doLayout();
  }

  public void setValue(int val)
  {
    model.setValue(val);
  }

  /**
   * Sets the progress enabled state of the dialog.
   *
   * @param Whether to enable the progress dialog.
   */
  public void setProgressEnabled(boolean val)
  {
    progressEnabled = val;
  }

  /**
   * Sets the color on the progress bar.
   *
   * @param The desired color.
   */
  public void setBarForeground(Color c)
  {
    progressBar.setForeground(c);
  }

  public void setBarBackground(Color c)
  {
    progressBar.setBackground(c);
  }

  /**
   * Indicates if the progressbar is enabled for use.
   *
   * @return is the progress bar enabled
   */
  public boolean isProgressEnabled()
  {
    return progressEnabled;
  }

  /**
   * Override the setVisible method of dialog to first determine if the
   * progressbar is set to be enabled.
   *
   * @param whether to set the dialog to visible or not.
   */
  public void setVisible(boolean val)
  {
    if(!isProgressEnabled()) return;

    // If we're setting it to go away, sleep for 2 secs to hold the dialog in
    // place, preventing a flash for small files.
    if(val==false)
      sleep(1000);

    super.setVisible(val);
  }

  /**
   * Increment the current value of the model by 1.
   */
  public void increment()
  {
    model.setValue(model.getValue()+1);
  }

  public void sleep(int msecs)
  {
    try
    {
      Thread.sleep(msecs);
    }
    catch(Exception e)
    {
    }
  }

  public Point getCenteredLocation(Frame frame)
  {
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    if (frameSize.height > screenSize.height)
      frameSize.height = screenSize.height;

    if (frameSize.width > screenSize.width)
      frameSize.width = screenSize.width;

    // don't put it 1/2 way down the screen ,settle for 1/3 since
    // it looks nicer ....
    int xoff = (screenSize.width - frameSize.width)/3;
    int yoff = (screenSize.height - frameSize.height)/3;
    return new Point(xoff,yoff);
  }

  public static void main(String args[])
  {
    int STEPS = 10;
    Frame frame = new Frame("Test ProgressBar");
    FileUploaderProgressBar bar = new FileUploaderProgressBar(frame);
    bar.setSteps(STEPS);
    bar.setMessage("A wee lovely little bar ...");
    bar.setBarBackground(Color.orange);
    bar.setBarForeground(Color.white);
    bar.setVisible(true);
    for(int i=0;i<=STEPS;i++)
    {
      try
      {
        bar.setValue(i);
        Thread.sleep(250);
      }
      catch(Exception e)
      {
      }
    }
    bar.setVisible(false);
    System.exit(0);
  }

}

